package com.oscar.base.util;

import com.google.common.io.FileWriteMode;
import com.google.common.io.Files;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.Base64;

public class FileUtils {

    /**
     * 图片文件转base64编码，image标签使用
     *
     * @param uploadPath config.getUploadPath()
     * @param fileName   xxx.jpg
     */
    public static String imageToBase64 (String uploadPath, String fileName) throws Exception {
        InputStream in = new FileInputStream(uploadPath + fileName);
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        byte[] buf = new byte[1024];
        int rc;
        while ((rc = in.read(buf)) != -1) {
            byteArrayOutputStream.write(buf, 0, rc);
        }
        byte[] encode = Base64.getEncoder().encode(byteArrayOutputStream.toByteArray());
        return "data:image/jpeg;base64," + new String(encode);
    }

    /**
     * 读取springboot中的resources下文件，不能读jar下的资源
     *
     * @param path txt/locations
     */
    public static String readResources (String path) throws IOException {
        Resource resource = new ClassPathResource(path);
        File file = resource.getFile();
        BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
        String line;
        StringBuilder sb = new StringBuilder();
        while ((line = br.readLine()) != null) {
            sb.append(line);
        }
        return sb.toString();
    }

    /**
     * 读取jar包下的资源
     *
     * @param path txt/locations
     */
    public static String readJarResources (String path) throws IOException {
        ClassPathResource cpr = new ClassPathResource(path);
        BufferedReader br = new BufferedReader(new InputStreamReader(cpr.getInputStream()));
        String line;
        StringBuilder sb = new StringBuilder();
        while ((line = br.readLine()) != null) {
            sb.append(line);
        }
        return sb.toString();
    }

    /**
     * 使用基本的HttpURLConnection类读取网络内容
     */
    public static void readHttpURLConnection (String httpUrl, File to) throws IOException {
        URL url = new URL(httpUrl);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        conn.setRequestMethod("GET");
        conn.setReadTimeout(60 * 1000);
        if (conn.getResponseCode() == HttpURLConnection.HTTP_OK) {
            InputStream inputStream = conn.getInputStream();

            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            byte[] buffer = new byte[10240];
            int len;
            while ((len = inputStream.read(buffer)) != -1) {
                bos.write(buffer, 0, len);
            }
            bos.writeTo(new FileOutputStream(to));
        }
    }

    /**
     * 下载图片到本地
     */
    public static void downloadPicture (String httpUrl, File to) {
        URL url;
        try {
            url = new URL(httpUrl);
            DataInputStream dataInputStream = new DataInputStream(url.openStream());

            FileOutputStream fileOutputStream = new FileOutputStream(to);
            ByteArrayOutputStream output = new ByteArrayOutputStream();

            byte[] buffer = new byte[1024];
            int length;

            while ((length = dataInputStream.read(buffer)) > 0) {
                output.write(buffer, 0, length);
            }
            fileOutputStream.write(output.toByteArray());
            dataInputStream.close();
            fileOutputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static String getFileName (String abstractPath) {
        final String[] split = abstractPath.split("/");
        return split[split.length - 1];
    }

    public static void save (String content, File to) {
        try {
            Files.write(content.getBytes(), to);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void append (String content, File to) {
        try {
            Files.asCharSink(to, Charset.forName("UTF-8"), FileWriteMode.APPEND).write(content);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 递归删除目录下的所有文件及子目录下所有文件
     *
     * @param dir 将要删除的文件目录
     * @return boolean Returns "true" if all deletions were successful.
     * If a deletion fails, the method stops attempting to
     * delete and returns "false".
     */
    public static boolean deleteDir (File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            //递归删除目录中的子目录下
            if (children != null) {
                for (String childrenDir : children) {
                    boolean success = deleteDir(new File(dir, childrenDir));
                    if (!success) {
                        return false;
                    }
                }
            }
        }
        // 目录此时为空，可以删除
        return dir.delete();
    }

    /**
     * 获取文件后缀名
     *
     * @return .jpg
     */
    public static String getSuffix (String fileName) {
        String[] split = fileName.split("\\.");
        return "." + split[split.length - 1];
    }

    /**
     * @return xxx .jpg
     */
    public static String[] splitFileName (String fileName) {
        final String[] split = fileName.split("\\.");
        StringBuilder name = new StringBuilder();
        String suffix = "";
        if (split.length >= 2) {
            for (int i = 0; i < split.length; i++) {
                if (i == split.length - 1) {
                    suffix = "." + split[i];
                } else {
                    name.append(split[i]);
                }
            }
            return new String[]{name.toString(), suffix};
        }
        return new String[]{fileName, ""};
    }
}


