package com.oscar.module.test.kafka.stream;

import cn.hutool.core.date.DateUtil;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.common.serialization.Serdes;
import org.apache.kafka.common.utils.Bytes;
import org.apache.kafka.streams.KafkaStreams;
import org.apache.kafka.streams.StreamsBuilder;
import org.apache.kafka.streams.StreamsConfig;
import org.apache.kafka.streams.Topology;
import org.apache.kafka.streams.kstream.*;
import org.apache.kafka.streams.state.Stores;
import org.apache.kafka.streams.state.WindowStore;

import java.time.Duration;
import java.time.ZoneId;
import java.util.Arrays;
import java.util.Properties;
import java.util.concurrent.CountDownLatch;

import static org.apache.kafka.streams.kstream.Materialized.as;
import static org.apache.kafka.streams.kstream.Suppressed.BufferConfig.unbounded;

/**
 * Desc :
 *
 * @author wl
 * Date:    2019/5/20 11:09
 * @version 1.0
 */
public class KafkaStreamApp {
    public static void main (String[] args) {
        System.out.println("stream-wordcount-application".hashCode()%50);
//        processorApi();
        DSL_Api1();
    }


    public static Properties getProperties(){
        Properties prop = new Properties();
        prop.put(StreamsConfig.APPLICATION_ID_CONFIG, "stream-wordcount-application");
        prop.put(StreamsConfig.BOOTSTRAP_SERVERS_CONFIG, "ubuntu:9092");

        //默认的序列化配置，可以被覆盖 Serializers/deserializers (serde)
        prop.put(StreamsConfig.DEFAULT_KEY_SERDE_CLASS_CONFIG, Serdes.String().getClass());
        prop.put(StreamsConfig.DEFAULT_VALUE_SERDE_CLASS_CONFIG, Serdes.String().getClass());

        prop.put(StreamsConfig.NUM_STREAM_THREADS_CONFIG,1);//默认1线程
        //setting offset reset to earliest so that we can re-run the demo code with the same pre-loaded data
        prop.put(ConsumerConfig.AUTO_OFFSET_RESET_CONFIG, "earliest");
        return prop;
    }
    public static void safeExit(KafkaStreams streams){
        //这个门栓厉害了，帮助我们释放资源
        final CountDownLatch latch = new CountDownLatch(1);

        // attach shutdown handler to catch control-c
        Runtime.getRuntime().addShutdownHook(new Thread("streams-wordcount-shutdown-hook") {
            @Override
            public void run () {
                streams.close();
                latch.countDown();//释放完资源后释放门栓
            }
        });

        try {
            streams.setUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {
                @Override
                public void uncaughtException (Thread t, Throwable e) {
                    //当stream线程由于意外终止时调用
                }
            });
            streams.start();
            //程序启动时阻塞这里，等接收到control -c 信号后执行jvm钩子，释放门栓然后执行退出
            latch.await();
        } catch (Throwable e) {
            System.exit(1);
        }
        System.exit(0);
    }
    private static void processorApi () {
        Topology topology = new Topology();
        topology.addSource("Source", SendWord.input_topic)  //添加数据源
                .addProcessor("Process", new SourceProcessor(), "Source")
                .addStateStore(Stores.keyValueStoreBuilder(
                        Stores.inMemoryKeyValueStore("ss_counts"),//也可以持久化 Stores.persistentKeyValueStore()
                        Serdes.String(),
                        Serdes.Integer()),
                        "Process");
        topology.addProcessor("PrintProcessor", new PrintProcessor(), "Process");
        topology.addSink("Sink", SendWord.output_topic, "Process");
        topology.addSink("sink_print_msg", "print_msg_topic", "PrintProcessor");

        System.out.println("+++++++++++++++++++++++");
        System.out.println(topology.describe());
        System.out.println("+++++++++++++++++++++++");

        final KafkaStreams streams = new KafkaStreams(topology, getProperties());
        safeExit(streams);
    }

    private static void DSL_Api () {
        final StreamsBuilder builder = new StreamsBuilder();


        //覆盖默认的序列化配置
        KStream<String, String> textLines = builder.stream(SendWord.input_topic, Consumed.with(Serdes.String(),Serdes.String()));
        KTable<String, Long> kTable = textLines
                // Create a new KStream
                .flatMapValues(value -> {
                    System.out.println("input record:"+value);
                    return Arrays.asList(value.toLowerCase().split("\\W+"));
                })
                .groupBy((key, word) -> {
                    System.out.printf("['%s','%s']\n",key,word);
                    return word;// select word as groupKey  返回什么就按照什么进行分组
                })
                // count the number of records in this new stream. use default store
                .count(as("counts-store"));

        // write this new kstream into another Kafka topic named stream_out_topic
        final KStream<String, Long> countStream = kTable.toStream();
        countStream.print(Printed.toSysOut());
        countStream.to(SendWord.output_topic, Produced.with(Serdes.String(), Serdes.Long()));


        final Topology topology = builder.build();
        System.out.println("+++++++++++++++++++++++");
        System.out.println(topology.describe());
        System.out.println("+++++++++++++++++++++++");
        KafkaStreams streams = new KafkaStreams(topology, getProperties());
        safeExit(streams);
    }
    private static void DSL_Api1 () {
        final StreamsBuilder builder = new StreamsBuilder();
        KStream<String, String> textLines = builder.stream(SendWord.input_topic, Consumed.with(Serdes.String(),Serdes.String()));

        textLines.groupBy((key, word) -> {
                    System.out.printf("['%s','%s']\n",key,word);
                    return word;
                })
                //20秒的时间窗口，支持延迟1秒到达的数据
                .windowedBy(TimeWindows.of(Duration.ofSeconds(20)).grace(Duration.ofMillis(1000)))
                .count(Materialized.<String, Long, WindowStore<Bytes, byte[]>>as("sessionized-aggregated-stream-store") /* state store name */
                        .withValueSerde(Serdes.Long())
                        .withKeySerde(Serdes.String()))
                //窗口关闭的时候放行
                .suppress(Suppressed.untilWindowCloses(unbounded()))
                .filter((windowedUserId, count)-> {
                    final String start = DateUtil.formatLocalDateTime(windowedUserId.window().startTime().atZone(ZoneId.systemDefault()).toLocalDateTime());
                    final String end = DateUtil.formatLocalDateTime(windowedUserId.window().endTime().atZone(ZoneId.systemDefault()).toLocalDateTime());
                    System.out.println(start+" 到 "+ end);
                    System.out.println("单词数大于3的为["+windowedUserId.key()+"],次数："+count);
                    System.out.println(count>3);
                    return count != null && count >3;
                })
        ;

        final Topology topology = builder.build();
        System.out.println("+++++++++++++++++++++++");
        System.out.println(topology.describe());
        System.out.println("+++++++++++++++++++++++");
        KafkaStreams streams = new KafkaStreams(topology, getProperties());
        safeExit(streams);
    }
}
