package com.oscar.module.test.zk;

import org.apache.zookeeper.*;
import org.apache.zookeeper.data.ACL;
import org.apache.zookeeper.data.Stat;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;

/**
 * Apache Curator项目封装了zk，可以一试
 */
public class ZkClient implements Watcher {
    private static final int SESSION_TIMEOUT = 30000;

    private ZooKeeper zk = null;
    private CountDownLatch countDownLatch = new CountDownLatch(1);//同步计数器

    @Override
    public void process (WatchedEvent event) {
        if (event.getState() == Event.KeeperState.SyncConnected) {//当客户端连接上zookeeper时
            countDownLatch.countDown();//计数器减一变为0，countDownLatch.await()唤醒
        }
    }

    /**
     * java api里创建zk客户端是异步的，为防止在客户端还未完成创建就被使用的情况，
     * 这里可以使用同步计时器，确保zk对象创建完成再被使用
     */
    public void connect (String hosts) throws IOException, InterruptedException {
        zk = new ZooKeeper(hosts, SESSION_TIMEOUT, this);
        countDownLatch.await();//阻塞程序继续执行
    }

    /**
     * 创建znode节点
     *
     * @throws KeeperException
     * @throws InterruptedException
     */
    public void create (ZNode zNode) throws KeeperException, InterruptedException {
        byte[] data = null;
        if (zNode.getData() != null) {
            data = zNode.getData().getBytes();
        }

        List<ACL> acl = null;
        if ("OPEN_ACL_UNSAFE".equals(zNode.getAcl())) {
            acl = ZooDefs.Ids.OPEN_ACL_UNSAFE;// 允许任何客户端对该znode进行读写
        } else if ("CREATOR_ALL_ACL".equals(zNode.getAcl())) {
            acl = ZooDefs.Ids.CREATOR_ALL_ACL;
        } else if ("READ_ACL_UNSAFE".equals(zNode.getAcl())) {
            acl = ZooDefs.Ids.READ_ACL_UNSAFE;
        }

        CreateMode createMode = null;
        if("PERSISTENT".equals(zNode.getCreateMode())){
            createMode = CreateMode.PERSISTENT;
        }else if("PERSISTENT_SEQUENTIAL".equals(zNode.getCreateMode())){
            createMode = CreateMode.PERSISTENT_SEQUENTIAL;
        }else if("EPHEMERAL".equals(zNode.getCreateMode())){
            createMode = CreateMode.EPHEMERAL;
        }else if("EPHEMERAL_SEQUENTIAL".equals(zNode.getCreateMode())){
            createMode = CreateMode.EPHEMERAL_SEQUENTIAL;
        }

        String createPath = zk.create(zNode.getPath(), data, acl, createMode);

    }

    /**
     * 如果提供的版本号和znode版本号一致才会删除这个znode，这样可以检测出对znode的修改冲突。
     * 通过将版本号设置为-1，可以绕过这个版本检测机制，无论znode的版本号是什么，都会直接将其删除
     */
    public void delete (String groupName) throws KeeperException, InterruptedException {
        String path = "/" + groupName;
        List<String> children = zk.getChildren(path, false);
        for (String child : children) {
            zk.delete(path + "/" + child, -1);
        }
        zk.delete(path, -1);
    }

    public void close () throws InterruptedException {
        if (zk != null) {
            try {
                zk.close();
            } catch (InterruptedException e) {
                throw e;
            } finally {
                zk = null;
                System.gc();
            }
        }
    }

    /**
     * 安全修改，类似CAS机制，会比较version是否过期
     * @param path
     * @param data
     * @param safe
     * @throws KeeperException
     * @throws InterruptedException
     */
    public void setData (String path,String data,boolean safe) throws KeeperException, InterruptedException {
        final Stat stat;
        try {
            stat = zk.exists(path, false);
            if (stat != null) {
                final int version = stat.getVersion();//当前znode的version
                System.out.println("此时等待别的客户端修改 " + path);
                zk.setData(path, data.getBytes(), version);//如果此时该znode被别的客户端修改，则版本号肯定不匹配，修改会失败
            }
        } catch (KeeperException | InterruptedException e) {
            if (e instanceof KeeperException.BadVersionException) {
                System.out.println("版本不一致，请重试");
            } else {
                throw e;
            }
        }
    }

    /**
     * 不管什么version，强制修改
     * @param path
     * @param data
     * @throws KeeperException
     * @throws InterruptedException
     */
    public void setData (String path,String data) throws KeeperException, InterruptedException {
        if(data != null)
            zk.setData(path,data.getBytes(),-1);
    }

    public boolean exists(String path) throws KeeperException, InterruptedException {
        return zk.exists(path, false) != null;
    }

    public List<String> ls (String path) throws KeeperException, InterruptedException {
        final List<String> children = zk.getChildren(path, false);
        for (String child : children) {
            System.out.println(child);
        }
        return children;
    }


    public String getData (String path) throws KeeperException, InterruptedException {
        final byte[] data = zk.getData(path, false, null);
        return new String(data);
    }

    public Map<String, Object> getData (String path, boolean returnStat) throws KeeperException, InterruptedException {
        Stat stat = new Stat();
        final byte[] data = zk.getData(path, false, stat);
        Map<String, Object> map = new HashMap<>();
        if (data == null) {
            map.put("data", "");
        } else {
            map.put("data", new String(data));
        }

        map.put("stat", stat);
        return map;
    }

    public boolean isClose(){
        return zk == null;
    }

    public static void main (String[] args) throws IOException, InterruptedException, KeeperException {
        ZkClient zkClient = new ZkClient();
        zkClient.connect("172.19.44.6:2181");
        zkClient.ls("/brokers");

        zkClient.close();
    }
}
